import os
import shutil
import tempfile

class tempdir(object):
    def make_temp_dir(self):
        self.tempdir = tempfile.mkdtemp()
        self.location = os.path.abspath(os.curdir)
        os.chdir(self.tempdir)

    def remove_temp_dir(self):
        os.chdir(self.location)
        try:
            shutil.rmtree(self.tempdir)
        except Exception as e:
            print(e)
            print(f"was not able to remove tempdir '{self.tempdir}'")

    def __enter__(self):
        self.make_temp_dir()

    def __exit__(self, type, value, traceback):
        if not isinstance(value, Exception):
            self.remove_temp_dir()



### "imports"
from example.classes import Data

### "other-imports"
import csv
import json

from example.utils import tempdir

### "csv-subclass"
class Csv(Data):
    """
    CSV type.
    """
    aliases = ['csv']

    def present(self):
        with tempdir():
            with open("dictionary.csv", "w") as f:
                headers = list(self.data[0].keys())
                writer = csv.DictWriter(f, headers)

                writer.writeheader()
                writer.writerows(self.data)

            with open("dictionary.csv", "r") as f:
                return f.read()
        
### "json-subclass"
class Json(Data):
    """
    JSON type.
    """
    aliases = ['json']

    def present(self):
        return json.dumps(self.data) 




class CashewException(Exception):
    def __init__(self, message):
        self.message = message

class InternalCashewException(CashewException):
    pass

class UserFeedback(CashewException):
    pass

class InactivePlugin(UserFeedback):
    def __init__(self, plugin_instance_or_alias):
        if isinstance(plugin_instance_or_alias, str):
            self.alias = plugin_instance_or_alias
        else:
            self.alias = plugin_instance_or_alias.alias

        self.message = "%s is inactive. Some additional software might need to be installed." % (self.alias)

class NoPlugin(UserFeedback):
    pass
